macroScript ExtrudePVert

category:"PolyMeshTools" 

tooltip:"Extrude Vertex"

buttontext:"Extrude Vertex"

icon:#("PolyMeshTool",1)



(



----------------------------------------------------------------------------------------

--   Script Name:	ExtrudePolyVertex 

--	 Version:		v0.2

--	 Started:       02/21/2001

--	 Last Modified: 02/25/2001

--   Description:	Extrude+Chamfer a PolyObject Vertex 

--	 Documentation: Visit http://www.gfxcentral.com/bobo/mxs4/polymeshtools

--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at

--   Idea by: 		Dave Dwire [Rainbow Studios], based on Nichimen's Nendo functionality

----------------------------------------------------------------------------------------

-- 0.12: Redraws Speed up

--       New workflow - click and release, adjust extusion, click again to confirm.

----------------------------------------------------------------------------------------







tool ExtrudePolyVertex numPoints:4

(



local start_view_Point 

local sel_verts

local all_verts =#()

local old_verts =#()

local min_length = 10^10



fn connect_all_verts verts_array =

(

disableSceneRedraw()

append verts_array verts_array[1]

ready_array =#()

for v = 1 to verts_array.count-1 do

(

for w = v+1 to verts_array.count do

(

faces1 = polyop.getFacesUsingVert $.baseobject v

faces2 = polyop.getFacesUsingVert $.baseobject w

for f in faces1 do 

(

if findItem faces2 f > 0 then 

	(

	if (verts_array[v]) != (verts_array[w]) and (findItem ready_array ((verts_array[v]) as string + "," +(verts_array[w]) as string) == 0) then

		(

		polyop.CreateEdge $.baseobject (verts_array[v]) (verts_array[w])

		append ready_array ((verts_array[v]) as string + "," +(verts_array[w]) as string)

		append ready_array ((verts_array[w]) as string + "," +(verts_array[v]) as string)

		exit

		)

	)--end if

)--end f loop

)--end w loop

)--end v loop

enableSceneRedraw()

)--end fn



fn get_normal_by_face vert =

(

faces = polyop.getFacesUsingVert $.baseobject vert

sum_normal = [0,0,0]

for f in faces do

(

sum_normal += polyop.getFaceNormal $.baseobject f

)--end f loop

normalize(sum_normal / faces.count)

)--end fn



fn create_new_polys =

(

disableSceneRedraw()

sel_verts = polyop.getVertSelection $.baseobject

old_verts = #()

all_verts =#()

for v in sel_verts do

(

vert_normal = get_normal_by_face v

append old_verts #(v,(polyop.getVert $.baseobject v),vert_normal)

new_verts = #()

length_array = #()

edges_list = polyop.getEdgesUsingVert $.baseobject v

for e in edges_list do

(

edge_vert = polyop.getEdgeVerts $.baseobject e

edge_length = length((polyop.getVert $.baseobject edge_vert[1])-(polyop.getVert $.baseobject edge_vert[2]))

append length_array edge_length 

div_edge = polyop.divideEdge $.baseobject e 0.5

append new_verts div_edge	

)--end e loop

connect_all_verts new_verts

deleteItem new_verts new_verts.count

old_pos = (polyop.GetVert $.baseobject v)

for i = 1 to new_verts.count do 

	(

	new_pos = (polyop.getVert $.baseobject new_verts[i])

	normal_vector = normalize (new_pos - old_pos)

	polyop.setVert $.baseobject new_verts[i] old_pos

	append all_verts #(new_verts[i], old_pos, normal_vector, length_array[i])

	)

)--end v loop

enableSceneRedraw()

sel_verts

)--end fn



fn ScaleVertices scale_array scale_amount=

(

disableSceneRedraw()

for v in scale_array do

(

if length(v[3]*scale_amount) < v[4] then

	(

	polyop.SetVert $.baseobject v[1] (v[2]+v[3]*scale_amount)

	)

)--end v loop

enableSceneRedraw()

)--end fn



fn ScaleOldVertices scale_array scale_amount =

(

for v in scale_array do

(

polyop.SetVert $.baseobject v[1] (v[2]+v[3]*scale_amount)

)--end v loop

)--end fn





on mousePoint click do 	

(

case click of

(

1: (

	create_new_polys ()

	start_view_Point = viewPoint

	)

2: ()

4: (#stop)

)--end case

)--end on



on mouseAbort click do  

	(

	--print "Abort!"

	max undo	--does not work 

	#stop

	)



on mouseMove click do  

	case click of 		

	(  		

	3: (

		ScaleVertices all_verts (abs(viewPoint.x-start_view_Point.x)*0.2)

		ScaleOldVertices old_verts (-(viewPoint.y-start_view_Point.y))

		)

	4: #stop  		

	)

)--end tool



pushprompt "Click and Drag Up/Down to Extrude, Left/Right to Chamfer"

if classof $.baseobject == Editable_Poly and subObjectLevel == 1 then starttool ExtrudePolyVertex 

)
