macroScript LiftFromEdge

category:"PolyMeshTools" 

tooltip:"Lift Polygon From Edge"

buttontext:"Lift Polygon From Edge"

icon:#("PolyMeshTool",2)

(



----------------------------------------------------------------------------------------

--   Script Name:	LeftPolyFromEdge

--	 Version:		v0.2 

--	 Started:       02/21/2001

--	 Last Modified: 02/25/2001  

--   Description:	One-Side-Extrusion of PolyObject Faces

--	 Documentation: Visit http://www.gfxcentral.com/bobo/mxs4/polymeshtools

--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at

--   Idea by: 		Dave Dwire [Rainbow Studios], based on Nichimen's Mirai functionality

----------------------------------------------------------------------------------------

--  0.2:	New meshing code using extrusion instead of detach+cap 

-- 			Redraws Speed Up

----------------------------------------------------------------------------------------



tool LeftPolyFromEdge numPoints:6

(

local start_view_Point 

local trans_matrix

local edge_verts_pos = #()

local edge_verts 

local start_view_Point 

local face_selection = 1

local faces_of_edge 

local current_verts

local verts_to_rotate



fn get_edge_to_lift =

(

edge_selection = polyop.getEdgeSelection $

selected_edge = #()

for e in edge_selection do append selected_edge e

if selected_edge.count == 1 then

(

sel_edge = selected_edge[1]

--format "Selected Edge %\n" sel_edge 

edge_verts = polyop.getEdgeVerts $ sel_edge 				--vertices of selected edge

edge_verts_pos = #()

edge_verts_pos[1]= (polyop.getVert $ edge_verts[1])

edge_verts_pos[2]= (polyop.getVert $ edge_verts[2])

faces_of_edge = polyop.getFacesUsingEdge $ #{sel_edge}		--faces that use the currently selected edge

return true

)

false

)--end fn





fn split_mesh_for_lifting =

(

disableSceneRedraw()

undo "LiftFromEdge" on

(

face_picked = undefined

pick_counter = 0

for f in faces_of_edge do 									--go through possible faces

(

pick_counter += 1

if pick_counter == face_selection then 

	(

	face_picked = f

	exit

	)

)



--format "Face to Lift From %\n" face_picked 

all_verts = polyop.GetFaceVerts $ face_picked

--format "All Vertices in Picked Face: %\n" all_verts 



polyop.ExtrudeFaces $ #{face_picked} 0



new_verts = polyop.GetFaceVerts $ face_picked

--format "New Vertices in Detached Face: %\n" new_verts 





old_vert_pos = #()

for v in all_verts do

(

append old_vert_pos (polyop.getVert $ v)

)



new_vert_pos = #()

for v in new_verts do

(

append new_vert_pos (polyop.getVert $ v)

)

--format "Old Verts. Pos: %\n" old_vert_pos

--format "New Verts. Pos: %\n" new_vert_pos





collapse_array = #{}

index_to_weld1 = FindItem old_vert_pos edge_verts_pos[1]

index_to_weld2 = FindItem new_vert_pos edge_verts_pos[1]



potential_edges = polyop.GetEdgesUsingVert $ all_verts[index_to_weld1]

for e in potential_edges do

(

both_verts = polyop.GetEdgeVerts $ e

if FindItem both_verts new_verts[index_to_weld2] > 0 then 

	(

	append collapse_array e

	exit

	)

)



index_to_weld1 = FindItem old_vert_pos edge_verts_pos[2]

index_to_weld2 = FindItem new_vert_pos edge_verts_pos[2]



potential_edges = polyop.GetEdgesUsingVert $ all_verts[index_to_weld1]

for e in potential_edges do

(

both_verts = polyop.GetEdgeVerts $ e

if FindItem both_verts new_verts[index_to_weld2] > 0 then 

	(

	append collapse_array e

	exit

	)

)



polyop.setEdgeSelection $ collapse_array

polyop.CollapseEdges $ collapse_array





current_verts = polyop.GetFaceVerts $ face_picked

current_vert_pos = #()

for v in current_verts do

(

append current_vert_pos (polyop.getVert $ v)

)

index_to_remove = FindItem current_vert_pos edge_verts_pos[1]

deleteItem current_vert_pos index_to_remove

deleteItem current_verts index_to_remove

index_to_remove = FindItem current_vert_pos edge_verts_pos[2]

deleteItem current_vert_pos index_to_remove

deleteItem current_verts index_to_remove



)--end undo

enableSceneRedraw()

)--end fn





fn define_lift_rotation =

(

rot_vector = normalize(edge_verts_pos[1]-edge_verts_pos[2])

trans_matrix = matrixfromnormal rot_vector

translate trans_matrix edge_verts_pos[2]



verts_to_rotate = #()

for f in current_verts do

(

in coordsys trans_matrix original_vert = polyop.getVert $ f

original_radius = sqrt (original_vert.x^2 + original_vert.y^2)

original_direction = atan (original_vert.y/original_vert.x)

append verts_to_rotate #(f,original_vert,original_radius,original_direction)

)

trans_matrix

)--end fn



fn rotate_vertices verts_to_rotate angle trans_matrix =

(

disableSceneRedraw()

if face_selection == 2 then 

	(

	rel_angle = degToRad(angle-verts_to_rotate[1][4])

	)

	else

	(

	angle = 180-angle

	rel_angle = degToRad(180-angle-verts_to_rotate[1][4])

	)

	

setStatusXYZ #angle [degToRad(angle),rel_angle,degToRad(angle-verts_to_rotate[1][4]-180)]



for v in verts_to_rotate do

(

in coordsys trans_matrix 

(

polyop.setVert $ v[1] ([v[3]*(cos angle),v[3]*(sin angle),v[2].z])

)

)--end v loop



enableSceneRedraw()

redrawViews()

)--end fn



fn select_face_by_position mpos =

(

test_count = 0

for f in faces_of_edge do test_count += 1

if test_count == 1 then 

(

mpos = -10

)



if mpos < 0 then

	(

	face_selection = 1

	face_sel = #{}

	cnt = 0

	for f in faces_of_edge do 

		(

		cnt += 1

		if cnt == 1 then append face_sel f

		)

	polyop.setFaceSelection $ face_sel 

	)

	else

	(

	face_selection = 2

	face_sel = #{}

	cnt = 0

	for f in faces_of_edge do 

		(

		cnt += 1

		if cnt == 2 then append face_sel f

		)

	if face_sel == #{} then face_selection = 1

	polyop.setFaceSelection $ face_sel 

	)

)--end fn





on mousePoint click do 	

(

case click of

(

1: (

	result = get_edge_to_lift()

	if result then

		(

		subObjectLevel = 4

		start_view_Point = viewPoint

		pushprompt "Drag Left/Right and Click to Select Poly To Lift"

		select_face_by_position ((viewPoint.x-start_view_Point.x))

		)

		else

		(

		pushprompt "Not a Single Edge!"

		subObjectLevel = 2

		#stop

		)

	)

3: (

	pushprompt "Drag Left/Right and Click to Define Lift Angle"

	start_view_Point = viewPoint

	split_mesh_for_lifting ()

	define_lift_rotation ()

	)

5: (

	subObjectLevel = 2

	#stop

	)

)--end case

)--end on



on mouseMove click do  

	(

	case click of 		

	(  		

	3: (

		select_face_by_position ((viewPoint.x-start_view_Point.x))

		max views redraw

		)

	4: (

		rotate_vertices verts_to_rotate ((viewPoint.x-start_view_Point.x)) trans_matrix

		)

	5: (

		subObjectLevel = 2

		pushprompt "LiftFromEdge Finished."

		#stop  		

		)

	)

	)

)--end tool





if selection.count == 1 then

(

if classof $.baseobject == Editable_Poly and subObjectLevel == 2 and $.modifiers.count == 0 then

	(

	pushprompt "Click To Start LiftFromEdge..."

	starttool LeftPolyFromEdge 

	)

)

)--end script
