macroScript LiftFromVert

category:"PolyMeshTools" 

tooltip:"Lift Polygon From Vertex"

buttontext:"Lift Polygon From Vertex"

icon:#("PolyMeshTool",3)

(



----------------------------------------------------------------------------------------

--   Script Name:	LeftPolyFromVertex

--	 Version:		v0.2 

--	 Started:       02/24/2001

--	 Last Modified: 02/25/2001

--   Description:	Extrusion of PolyObject Faces with a single vertex locked down

--	 Documentation: Visit http://www.gfxcentral.com/bobo/mxs4/polymeshtools

--	 Code by:		Borislav Petrov [Boboland]		bobo@email.archlab.tuwien.ac.at

--   Idea by: 		Dave Dwire [Rainbow Studios], based on Nichimen's Mirai functionality

----------------------------------------------------------------------------------------

--  0.2: Redraws Speed Up

----------------------------------------------------------------------------------------





tool LeftPolyFromVertex numPoints:6

(

local start_view_Point 

local trans_matrix

local start_view_Point 

local face_selection = 1

local faces_of_vert

local current_verts

local verts_to_rotate

local lift_vert_pos 

local edges_for_tm



fn get_vert_to_lift =

(

vert_selection = polyop.getVertSelection $

selected_vert = #()

for v in vert_selection do append selected_vert v

if selected_vert.count == 1 then

(

sel_vert = selected_vert[1]

--format "Selected Vert %\n" sel_vert 

faces_of_vert= polyop.getFacesUsingVert $ #{sel_vert}		--faces that use the currently selected vertex

lift_vert_pos = (polyop.getVert $ sel_vert)

return true

)

false

)--end fn





fn split_mesh_for_lifting =

(

disableSceneRedraw()

undo "LiftFromVertex" on

(

face_picked = undefined

pick_counter = 0

for f in faces_of_vert do 									--go through possible faces

(

pick_counter += 1

if pick_counter == face_selection then 

	(

	face_picked = f

	exit

	)

)



--format "Face to Lift From %\n" face_picked 

all_verts = polyop.GetFaceVerts $ face_picked

--format "All Vertices in Picked Face: %\n" all_verts 

polyop.ExtrudeFaces $ #{face_picked} 0



new_verts = polyop.GetFaceVerts $ face_picked

--format "New Vertices in Detached Face: %\n" new_verts 





old_vert_pos = #()

for v in all_verts do

(

append old_vert_pos (polyop.getVert $ v)

)



new_vert_pos = #()

for v in new_verts do

(

append new_vert_pos (polyop.getVert $ v)

)

--format "Old Verts. Pos: %\n" old_vert_pos

--format "New Verts. Pos: %\n" new_vert_pos



collapse_array = #{}

index_to_weld1 = FindItem old_vert_pos lift_vert_pos

index_to_weld2 = FindItem new_vert_pos lift_vert_pos



potential_edges = polyop.GetEdgesUsingVert $ all_verts[index_to_weld1]

for e in potential_edges do

(

both_verts = polyop.GetEdgeVerts $ e

if FindItem both_verts new_verts[index_to_weld2] > 0 then 

	(

	append collapse_array e

	exit

	)

)



--print collapse_array

polyop.setEdgeSelection $ collapse_array

polyop.CollapseEdges $ collapse_array



current_edges = polyop.GetFaceEdges $ face_picked

edges_for_tm = #()

for e in current_edges do

(

edge_verts = polyop.getEdgeVerts $ e

if polyop.GetVert $ edge_verts[1] == lift_vert_pos then

	(

	append edges_for_tm e

	)

if polyop.GetVert $ edge_verts[2] == lift_vert_pos then

	(

	append edges_for_tm e

	)

)--end e loop



--print edges_for_tm



current_verts = polyop.GetFaceVerts $ face_picked

current_vert_pos = #()

for v in current_verts do

(

append current_vert_pos (polyop.getVert $ v)

)

index_to_remove = FindItem current_vert_pos lift_vert_pos

deleteItem current_vert_pos index_to_remove

deleteItem current_verts index_to_remove

--Format "Vertices to Rotate: %\n" current_vert_pos



)--end undo

enableSceneRedraw()

)--end fn





fn define_lift_rotation =

(

edge_points = polyop.GetEdgeVerts $ edges_for_tm[1]

vert1 = polyop.GetVert $ edge_points[1]

vert2 = polyop.GetVert $ edge_points[2]

mid1 = (vert1+vert2)/2



edge_points = polyop.GetEdgeVerts $ edges_for_tm[2]

vert1 = polyop.GetVert $ edge_points[1]

vert2 = polyop.GetVert $ edge_points[2]

mid2 = (vert1+vert2)/2



rot_vector = normalize(mid1-mid2)

trans_matrix = matrixfromnormal rot_vector

translate trans_matrix lift_vert_pos



verts_to_rotate = #()

for f in current_verts do

(

in coordsys trans_matrix original_vert = polyop.getVert $ f

original_radius = sqrt (original_vert.x^2 + original_vert.y^2)

original_direction = atan (original_vert.y/original_vert.x)

append verts_to_rotate #(f,original_vert,original_radius,original_direction)

)

trans_matrix

)--end fn



fn rotate_vertices verts_to_rotate angle trans_matrix =

(

disableSceneRedraw()

if face_selection == 2 then 

	(

	rel_angle = degToRad(angle-verts_to_rotate[1][4])

	)

	else

	(

	angle = 180-angle

	rel_angle = degToRad(180-angle-verts_to_rotate[1][4])

	)

	

setStatusXYZ #angle [degToRad(angle),rel_angle,degToRad(angle-verts_to_rotate[1][4]-180)]



for v in verts_to_rotate do

(

in coordsys trans_matrix 

(

polyop.setVert $ v[1] ([v[3]*(cos angle),v[3]*(sin angle),v[2].z])

)

)--end v loop

enableSceneRedraw()

redrawViews()

)--end fn



fn select_face_by_position mpos =

(

test_count = 0

for f in faces_of_vert do test_count += 1

if test_count == 1 then 

(

mpos = -50

)

counter = 0

for m = -200 to 199 by (400.0/test_count) do

(

counter += 1

if mpos > m and mpos < m+(400.0/test_count)  then

	(

	face_selection = counter

	exit

	)--end if

)--end m loop	

	face_sel = #{}

	cnt = 0

	for f in faces_of_vert do 

		(

		cnt += 1

		if cnt == face_selection then append face_sel f

		)

	polyop.setFaceSelection $ face_sel 

	if face_sel == #{} then face_selection = 1

	

)--end fn





on mousePoint click do 	

(

case click of

(

1: (

	result = get_vert_to_lift()

	if result then

		(

		subObjectLevel = 4

		start_view_Point = viewPoint

		pushprompt "Drag Left/Right and Click to Select Poly To Lift"

		select_face_by_position ((viewPoint.x-start_view_Point.x))

		)

		else

		(

		pushprompt "Not a Single Vertex!"

		subObjectLevel = 1

		#stop

		)

	)

3: (

	pushprompt "Drag Left/Right and Click to Define Lift Angle"

	start_view_Point = viewPoint

	split_mesh_for_lifting ()

	define_lift_rotation ()

	)

5: (

	subObjectLevel = 1

	#stop

	)

)--end case

)--end on



on mouseMove click do  

	(

	case click of 		

	(  		

	3: (

		select_face_by_position ((viewPoint.x-start_view_Point.x))

		max views redraw

		)

	4: (

		rotate_vertices verts_to_rotate ((viewPoint.x-start_view_Point.x)) trans_matrix

		)

	5: (

		subObjectLevel = 1

		pushprompt "LiftFromVertex Finished."

		#stop  		

		)

	)

	)

)--end tool





if selection.count == 1 then

(

if classof $.baseobject == Editable_Poly and subObjectLevel == 1 and $.modifiers.count == 0 then

	(

	pushprompt "Click To Start LiftFromVertex..."

	starttool LeftPolyFromVertex 

	)

)

)--end script
